//	AsyncAlert.cpp

#include "AsyncAlert.h"
#include <colors.h>
#include <StLocker.h>

#include <TextView.h>
#include <Button.h>
#include <Window.h>
#include <Beep.h>


#define BUTTON_PRESSED 'bpr!'


static BRect defaultRect(200,150, 600,300);

AsyncAlert::AsyncAlert(
	const char *	message,
	BMessage *		reply,
	BHandler *		target,
	const char *	btn1,
	const char *	btn2,
	const char *	btn3) :
	BWindow(defaultRect, "Alert", B_MODAL_WINDOW, 
			B_NOT_RESIZABLE | B_NOT_ZOOMABLE | B_NOT_CLOSABLE)
{
	StLocker<BWindow> lock(this);

	fReply = reply;
	fTarget = target;
	if (reply && !target) {
		delete fReply;
		fReply = NULL;
	}
	BRect bounds = Bounds();
	BView *bg = new BView(bounds, "bg", B_FOLLOW_ALL, B_WILL_DRAW);
	AddChild(bg);
	bg->SetViewColor(ltGray);
	BRect frm = bounds;
	frm.left += 50;
	frm.right -= 10;
	frm.top += 5;
	frm.bottom -= 40;
	BRect tx = frm;
	tx.OffsetTo(B_ORIGIN);
	fMessage = new BTextView(frm, "message", tx, B_FOLLOW_ALL, B_WILL_DRAW);
	bg->AddChild(fMessage);
	fMessage->SetFontName("Emily");
	fMessage->SetFontSize(9);
	fMessage->SetWordWrap(true);
	fMessage->MakeEditable(false);
	fMessage->SetViewColor(ltGray);
	fMessage->SetLowColor(ltGray);
	fMessage->SetText(message);

	if (btn1 == NULL)
		btn1 = "OK";
	BButton *btn = MakeButton(bounds, 1, btn1);
	bg->AddChild(btn);
	SetDefaultButton(btn);

	if (btn2) {
		btn = MakeButton(bounds, 2, btn2);
		bg->AddChild(btn);
	}

	if (btn3) {
		btn = MakeButton(bounds, 3, btn3);
		bg->AddChild(btn);
	}
}


void
AsyncAlert::Go()
{
	beep();
	Lock();
	Resize();
	Activate();
	Show();
	Unlock();
}


bool
AsyncAlert::QuitRequested()
{
	if (fTarget && fReply)
		fTarget->Looper()->PostMessage(fReply, fTarget);
	return TRUE;
}


void
AsyncAlert::MessageReceived(
	BMessage *message)
{
	switch (message->what) {
	case BUTTON_PRESSED:
		if (fReply && fTarget) {
			fReply->AddLong("answer", message->FindLong("button"));
			fTarget->Looper()->PostMessage(fReply, fTarget);
		}
		Quit();
		break;
	default:
		inherited::MessageReceived(message);
		break;
	}
}


BButton *
AsyncAlert::MakeButton(
	const BRect &	bounds,
	long			index,
	const char *	title)
{
	BRect btnr = bounds;
	btnr.InsetBy(10, 10);
	btnr.top = btnr.bottom-20.0;
	btnr.left = btnr.right-70.0;
	btnr.OffsetBy(-80*(index-1), 0);
	BMessage *msg = new BMessage(BUTTON_PRESSED);
	msg->AddLong("button", index);
	AddShortcut(*title, 0, new BMessage(msg));
	BButton *btn = new BButton(btnr, "", title, msg, B_FOLLOW_BOTTOM | B_FOLLOW_RIGHT);
	return btn;
}


void
AsyncAlert::Resize()
{
	BRect tx = fMessage->TextRect();
	BRect r2 = fMessage->Bounds();
	float delta_y = tx.Height()-r2.Height();
	ResizeBy(0, delta_y);
}
