//	ContainerWindow.h
//
//	Window that can contain one PluginView
//
//	Jon Watte (h+@metrowerks.com) 96-09-29


#pragma once


#include <Window.h>
#include <MList.h>

class PluginView;
class DStream;
class EditorPanel;
class AsyncAlert;


class ContainerWindow : 
	public BWindow
{
public:
								ContainerWindow(
									BRect			area,
									const char *	title,
									window_type		type,
									ulong			flags);
								~ContainerWindow();
virtual	void					SetupWindow();
virtual	BMenu *					MakeFileMenu();
virtual	long					OpenFile(
									record_ref		ref,
									ulong			type,
									const char *	mime);
virtual	long					NewFile();

		void					MessageReceived(
									BMessage *		message);
		bool					QuitRequested();
		void					SaveRequested(
									record_ref		directory,
									const char *	name);
		void					MenusWillShow();
		void					Activate(
									bool			toActive = TRUE);

protected:

		BMenu *					fFileMenu;
		bool					fHasFile;
		record_ref				fFileRef;
		PluginView *			fTheView;
		BMenuBar *				fMenuBar;
		bool					fDirty;
		EditorPanel *			fEditorPanel;
		AsyncAlert *			fRevertAlert;
		AsyncAlert *			fCloseAlert;
		ulong					fDocType;
		char *					fDocMIME;
		MList<BMenu *>			fLostMenus;

//	PluginView message hooks

virtual	void					DoFocusMenus(
									BMessage *		message,
									PluginView *	forView);
virtual	void					RemoveLostMenus();
virtual	void					DoLostFocus(
									BMessage *		message,
									PluginView *	forView);
virtual	void					DoDoneEditting(
									PluginView *	forView,
									bool			sendSave);
virtual	void					DoPleaseSave(
									PluginView *	forView);
virtual	void					DoDataChanged(
									PluginView *	forView);
virtual	void					DoSelectFrame(
									PluginView *	forView);

//	Regular commands

virtual	void					DoPrint(
									BMessage *		message);
virtual	void					DoSave(
									BMessage *		message);
virtual	void					DoSaveAs(
									BMessage *		message);
virtual	void					DoRevert(
									BMessage *		message);

//	other message handlers

virtual	void					DoEditorChosen(
									BMessage *		message);
virtual	void					DoCloseReply(
									BMessage *		message);
virtual	void					DoReallyRevert(
									BMessage *		message);

};

