//	EditorPanel.cpp

#include <Debug.h>

#include "EditorPanel.h"
#include "PluginEditors.h"
#include "PluginView.h"
#include <StLocker.h>

#include <colors.h>
#include <Button.h>
#include <Message.h>
#include <stdlib.h>


#define EDITOR_SELECTED 'ese!'


static BRect area(150,100, 450,350);

EditorPanel::EditorPanel(
	const char *	title,
	BMessage *		reply,
	BHandler *		target,
	ulong			type,
	const char *	mime) :
	BWindow(area, title, B_TITLED_WINDOW, 0)
{
	StLocker<BWindow> lock(this);

	ASSERT(reply != NULL);
	ASSERT(target != NULL);
	BRect bounds = Bounds();
	BView *bg = new BView(bounds, "bg", B_FOLLOW_ALL, B_WILL_DRAW);
	AddChild(bg);
	bg->SetViewColor(ltGray);
	BRect lv = bounds;
	lv.InsetBy(5,5);
	lv.bottom -= 35;
	lv.right -= B_V_SCROLL_BAR_WIDTH;
	fReply = reply;
	fTarget = target;
	fList = new EditorList(lv, B_FOLLOW_ALL, B_WILL_DRAW | B_NAVIGABLE);
	bg->AddChild(fList);
	fList->SetInvocationMessage(new BMessage(EDITOR_SELECTED));
	BRect br = bounds;
	br.InsetBy(5, 5);
	br.left = br.right-70;
	br.top = br.bottom-20;
	BButton *btn = new BButton(br, "ok", "OK", new BMessage(EDITOR_SELECTED),
		B_FOLLOW_RIGHT | B_FOLLOW_BOTTOM);
	bg->AddChild(btn);
	SetDefaultButton(btn);
	fType = type;
	fMime = mime ? strdup(mime) : NULL;
}


EditorPanel::~EditorPanel()
{
	if (fMime)
		free(fMime);
}


void
EditorPanel::Go()
{
	Lock();
	fList->ListEditors(fType, fMime);
	Activate();
	Show();
	fList->MakeFocus(TRUE);
	Unlock();
}


bool
EditorPanel::QuitRequested()
{
	fReply->AddLong("editor", fList->GetSelectedEditor());
	fTarget->Looper()->PostMessage(fReply, fTarget);
	return true;
}


void
EditorPanel::MessageReceived(
	BMessage *		message)
{
	switch (message->what) {
	case EDITOR_SELECTED:
		fReply->AddLong("editor", fList->GetSelectedEditor());
		fTarget->Looper()->PostMessage(fReply, fTarget);
		Quit();
		break;
	default:
		inherited::MessageReceived(message);
		break;
	}
}




struct EItem {
	char *		name;
	EDITID		editor;
				EItem(
					const char *n,
					EDITID id) {
					name = strdup(n);
					editor = id;
				}
				~EItem() {
					free(name);
				}
};


EditorList::EditorList(
	BRect			area,
	ulong			resize,
	ulong			flags) :
	BListView(area, "editors", resize, flags | B_FRAME_EVENTS)
{
	//	do nothing
}


EditorList::~EditorList()
{
	for (int ix=CountItems()-1; ix>=0; ix--) {
		EItem *i = (EItem *)ItemAt(ix);
		delete i;
	}
}


void
EditorList::DrawItem(
	BRect			updateArea,
	long			index)
{
	BRect rct = ItemFrame(index);
	DrawString(((EItem *)ItemAt(index))->name, BPoint(5, rct.bottom-3));
}


void
EditorList::ListEditors(
	ulong			type,
	const char *	mime)
{
	EDITID * editors = NULL;
	long count = 0;
	long err = EDITGetAllEditors(editors, count);
	ASSERT(err == B_NO_ERROR);
	if (err)
		return;
	for (int ix=0; ix<count; ix++) {
		const char *name;
		const char *info;
		long version;
		if (EDITGetEditorInfo(editors[ix], name, info, version))
			continue;
		EDITInfo *formats = NULL;
		long fmts = 0;
		if (EDITGetEditorFormats(editors[ix], formats, fmts))
			continue;
		bool useit = true;
		if (type || mime) {
			useit = false;
			for (int iy=0; iy<fmts; iy++) {
				if (type && (formats[iy].formatType == type))
					useit = true;
				else
				if (mime && !strcmp(formats[iy].MIMEName, mime))
					useit = true;
			}
		}
		if (useit) {
			EItem *item = new EItem(name, editors[ix]);
			AddItem(item);
		}
	}
	delete[] editors;
}


EDITID
EditorList::GetSelectedEditor()
{
	long sel = CurrentSelection();
	if (sel < 0)
		return EDIT_NO_EDITOR;
	return ((EItem *)ItemAt(sel))->editor;
}


