//	PluginApp.cpp

#include "PluginApp.h"

#include "PluginEditors.h"
#include "Datatypes.h"
#include "AsyncAlert.h"
#include "ContainerWindow.h"


const ulong SIGNATURE = 'PLUG';



void
main()
{
	PluginApp app(SIGNATURE);
	long err = DATAInit(SIGNATURE);
	if (!err)	
		err = EDITInit(SIGNATURE);
	if (err) {
		char str[200];
		sprintf(str, "Could not initialize libraries.\n\nError: %08x",
			err);
		AsyncAlert *alert = new AsyncAlert(str, new BMessage(B_QUIT_REQUESTED),
			&app);
		app.SetShowOpen(false);
		alert->Go();
	}
	app.Run();
	EDITShutdown();
	DATAShutdown();
}



PluginApp::PluginApp(
	ulong sig) :
	BApplication(sig)
{
	fShowOpen = true;
	SetPulseRate(500000);
	SetMainMenu(MakeMainMenu());
}


bool
PluginApp::QuitRequested()
{
	if (IsFilePanelRunning())
		CloseFilePanel();
	return inherited::QuitRequested();
}


void
PluginApp::MessageReceived(
	BMessage *message)
{
	switch (message->what) {
	case CMD_NEW:
		NewDocument();
		break;
	case CMD_OPEN:
		RunFilePanel();
		break;
	default:
		inherited::MessageReceived(message);
		break;
	}
}


void
PluginApp::RefsReceived(
	BMessage *message)
{
	long cnt = 0;
	ulong type = 0;
	if (!message->GetInfo("refs", &type, &cnt))
		return;
	for (int ix=0; ix<cnt; ix++) {
		record_ref ref = message->FindRef("refs", ix);
		OpenDocument(ref);
	}
}


void
PluginApp::Pulse()
{
	if (!CountWindows() && !IsFilePanelRunning()) {
		if (fShowOpen) {
			RunFilePanel();
			fShowOpen = false;
		} else {
			PostMessage(B_QUIT_REQUESTED);
		}
	}
}


void
PluginApp::SetShowOpen(
	bool show)
{
	fShowOpen = show;
}


BPopUpMenu *
PluginApp::MakeMainMenu()
{
	BPopUpMenu *menu = new BPopUpMenu("main", false, false);
	BMenuItem *item = NULL;

	item = new BMenuItem("About PlugApp", new BMessage(B_ABOUT_REQUESTED));
	menu->AddItem(item);

	menu->AddSeparatorItem();

	item = new BMenuItem("New", new BMessage(CMD_NEW), 'N');
	menu->AddItem(item);

	item = new BMenuItem("Open", new BMessage(CMD_OPEN), 'O');
	menu->AddItem(item);

	menu->AddSeparatorItem();

	item = new BMenuItem("Quit", new BMessage(B_QUIT_REQUESTED), 'Q');
	menu->AddItem(item);

	return menu;
}


static int sWindowCount = 0;

void
PluginApp::NewDocument()
{
	BRect rct(80,20, 480,320);
	rct.OffsetBy((sWindowCount&7)*30, (sWindowCount&3)*40);
	sWindowCount++;
	ContainerWindow *window = new ContainerWindow(rct, "Untitled", B_TITLED_WINDOW, 0);
	window->SetupWindow();
	window->NewFile();
}


void
PluginApp::OpenDocument(
	record_ref ref)
{
	ulong type, creator;
	{
		BFile file;
		if (file.SetRef(ref))
			return;
		file.GetTypeAndApp(&type, &creator);
	}
	BRect rct(80,20, 480,320);
	rct.OffsetBy((sWindowCount&7)*30, (sWindowCount&3)*40);
	sWindowCount++;
	ContainerWindow *window = new ContainerWindow(rct, "Untitled", B_TITLED_WINDOW, 0);
	window->SetupWindow();
	window->OpenFile(ref, type, NULL);
}

