//	PluginEditors.h
//
//	Header file for using plugin editors
//
//	Jon Watte (h+@metrowerks.com) 960926

#pragma once


#include <Datatypes.h>

#include "PluginProtocol.h"


class PluginView;
struct EDITInfo;


#pragma export on


typedef unsigned long EDITID;

enum EDIT_ERROR {
	EDIT_BASE_ERROR		=	0xEDED0000,		//	generic error for PluginEditors.lib
	EDIT_NO_EDITOR,							//	there is no editor for this type of data
	EDIT_NOT_INITIALIZED,					//	EDITInit() has not been successfully called
	EDIT_BAD_PARAMETERS,					//	generic "garbage in" error
	EDIT_NO_PREVIOUS_STREAM,				//	return if Output(NULL) is called when no previous Input()
	EDIT_CANNOT_EDIT						//	return if called with SetCanEdit(TRUE) for viewer
};


#define EDIT_CURRENT_VERSION	100
#define EDIT_MIN_VERSION		100


extern "C" {

			//	you need to initialize the library to use it
			//	you should also shutdown it before quitting

	extern	const char *EDITVersion(	//	returns version string
							long & outCurVersion,	//	current version spoken
							long & outMinVersion);	//	minimum version understood

	extern	long		EDITInit(		//	establish connection
							ulong				mySignature,
							BQuery *			editors = NULL);

	extern	long		EDITShutdown();	//	don't want to talk anymore

			//	these functions call through to the translators
			//	when wantType is not 0, will only take into consideration 
			//	handlers that can read input data and produce output data

	extern	long		EDITFindEditor(	//	find out what can edit something
							bool				inForEditing,
							ulong				inDataType,//	kAnyType for MIME
							const char *		inMIMEType,
							EDITID &			outEditor);

	extern	long		EDITEditorFromFile(//	find EDITID that file is
							record_ref			inFile,
							EDITID &			outEditor,
							bool				inTryToLoad = FALSE);//	if true, load this editor

	extern	long		EDITGetAllEditors(//	find all handler IDs
							EDITID * &			outList,//	call delete[] when done
							long &				outCount);

	extern	long		EDITGetEditorFormats(//	find formats for an editor
							EDITID				inEditor,
							const EDITInfo * &	outFormats,//	don't write contents!
							long &				outNumFormats);

	extern	long		EDITGetEditorInfo(
							EDITID				inEditor,
							const char * &		outName,
							const char * &		outInfo,
							ulong &				outVersion);

	extern	long		EDITMakePlugin(	//	create PluginView subclass
							EDITID				inEditor,
							const BRect &		inArea,
							const char *		inName,
							ulong				inResize,
							ulong				inFlags,
							BHandler *			inContainer,
							BMessage *			inExtension,
							PluginView * &		outView);

	extern	long		EDITSetPreferredEditor(
							EDITID				inEditor,
							bool				inForEditting,
							ulong				inForType,
							const char *		inForMIME);

	extern	long		EDITGetPreferredEditor(
							bool				inForEditting,
							ulong				inForType,
							const char *		inForMIME,
							EDITID &			outEditor);

}


//	This setting name will return a list of record_refs for 
//	folders that contain editors. If you want to change 
//	this list at runtime and have the settings take effect, 
//	please close and reopen the editors library.

extern const char editorEditorFolders[];


#pragma export reset

