//	PluginView.cpp
//
//	Base class for all plugin editors
//
//	Jon Watte (h+@metrowerks.com) 960927


#include "PluginView.h"
#include "PluginProtocol.h"
#include <Debug.h>


PluginView::PluginView(
	BRect			area, 
	const char *	name,
	ulong			resize, 
	ulong			flags, 
	BHandler *		container,
	BMessage *		extension) :
	BView(area, name, resize, flags)
{
	ASSERT(fContainer != NULL);
	fContainer = container;
}


PluginView::~PluginView()
{
	//	do nothing
}


void
PluginView::MessageReceived(
	BMessage *		message)
{
	BMessage *reply = NULL;
	BMessage **param = &reply;
	if (!message->IsSourceWaiting()) {
		param = NULL;
	}
	if (ExtensionMessage(message, param)) {
		if (param && param) {
			ASSERT(*param != NULL);
			message->SendReply(*param);
		}
		return;
	}
	inherited::MessageReceived(message);
}


bool
PluginView::ExtensionMessage(
	BMessage *		inMessage,
	BMessage **		outMessage)
{
	return FALSE;
}


void
PluginView::MenusWillShow()
{
	//	do nothing
}


//	what's your preferred screen size?
BRect
PluginView::GetScreenArea()
{
	//	whatever is there is fine by me
	return Bounds();
}


//	what's your total data size? (for printing etc)
BRect
PluginView::GetImageArea()
{
	//	we don't scroll, and we always print all we're showing
	return Bounds();
}


//	you can do pagination, too! To be able to print single records, 
//	or whole lines, or whatever that may not be an even multiple of 
//	the available print space
long
PluginView::GetNumPages(
	const BRect & areaToPrint)
{
	BRect toPrint = GetImageArea();
	long numPages = ceil(toPrint.Height()/areaToPrint.Height());
	if (numPages < 1)
		numPages = 1;
	return numPages;
}


//	determine what BRect we want to be sent for this page
//	it must be smaller than or equal in size to the forArea
//	area.
BRect
PluginView::GetPageRect(
	long ix,
	const BRect & forArea)
{
	BRect ret = forArea;
	ret.OffsetBy(0, forArea.Height()*ix);
	return ret;
}


		//	Just send the message
long
PluginView::PostToContainer(
	BMessage *		inMessage)
{
	return fContainer->Looper()->PostMessage(inMessage, fContainer);
}


		//	We got focus - tell container about our menus.
		//	Menus should be passed as BList of BMenu objects.
		//	Views have to make arrangements to get their menu 
		//	commands on their own; typically by calling 
		//	SetTarget() on its menu items.
long
PluginView::FocusMenus(
	bool			gotFocus,
	const BList &	inMenus)
{
	BMessage *message = new BMessage(gotFocus ? VIEW_FOCUS_MENUS : VIEW_LOST_FOCUS);
	message->AddObject("view", this);
	for (int ix=0; ix<inMenus.CountItems(); ix++) {
		message->AddObject("menus", (BMenu *)inMenus.ItemAt(ix));
	}
	return PostToContainer(message);
}


//	The user indicated he's "done editting" the content
//	Note that the container may decide on this without view 
//	intervention.
long
PluginView::DoneEditting(
	bool doSave)
{
	BMessage *message = new BMessage(VIEW_DONE_EDITTING);
	message->AddObject("view", this);
	message->AddBool("save", doSave);
	return PostToContainer(message);
}


//	The user indicated he wants to "save" the data
long
PluginView::PleaseSave()
{
	BMessage *message = new BMessage(VIEW_PLEASE_SAVE);
	message->AddObject("view", this);
	return PostToContainer(message);
}


//	Send this when the user starts editing data after 
//	it's clean (saved or freshly loaded)
long
PluginView::DataChanged()
{
	BMessage *message = new BMessage(VIEW_DATA_CHANGED);
	message->AddObject("view", this);
	return PostToContainer(message);
}


//	Tell the container to select our frame to manipulate 
//	it (resize etc). Can be used by "Select Frame" command.
long
PluginView::SelectFrame()
{
	BMessage *message = new BMessage(VIEW_SELECT_FRAME);
	message->AddObject("view", this);
	return PostToContainer(message);
}
